/* 
talkthru.c
Michael G. Morrow, 2005
talkthru demo program 
*/                                  

#include <c6x.h>
#include "..\c6x11dsk.h"   
#include "..\support.h"
#include "..\winDSK6_def.h"
#include "hpi.h"

#include <math.h>

// define the COMPLEX structure 
// floats used as DSP target is intended; could be double on PC
typedef struct {
    float real, imag;
} COMPLEX;

// function prototypes 
void fft_c(int n, COMPLEX *x, COMPLEX *W);
void init_W(int n, COMPLEX *W);

#define N BUFFER_LENGTH  

COMPLEX x[N];

float window[N];
    
COMPLEX W[N];  // for twiddle factors

int main()
{    
  	int i;
  	
  	CSR=0x100;			        	       /* disable all interrupts  */ 
  	IER=0;

    // initialize real and imaginary parts of the twiddle factors 
    init_W(N, W);
    
    // calculate triangular window coefficients
    for(i = 0;i < ((BUFFER_LENGTH+1)/2);i++)
    	window[i] = window[(BUFFER_LENGTH-1)-i] = i * (1/((float)((BUFFER_LENGTH+1)/2)));

	while(HPI_Block.CODEC < 0) /* wait for host to set codec type */
		;
	
	HPI_Block.SampleFreq = InitSerialComm(HPI_Block.CODEC); // init codec

  	ICR = 0xffff;       /* clear all pending interrupts */
  	CSR |= 1;           /* set GIE */
  	
  	while(1) { /* main stalls here */
		if(HPI_Block.Index >= BUFFER_LENGTH) { // buffer is ready to process
  			for(i = 0;i < BUFFER_LENGTH;i++) { // fill buffer for FFT
  				// sum two channels
  				x[i].real = (short)(Buffer[i] & 0xFFFF) + (short)(Buffer[i] >> 16);
//   				x[i].real *= window[i]; // apply window
  				x[i].imag = 0;
  			}
 	 		HPI_Block.Index = 0; 		// can start storing data again now
  			
	    	fft_c(N, x, W);				// calculate the FFT of x[N] 
	    	
   			for(i = 0;i < BUFFER_LENGTH;i++) { // compute magnitude spectrum
   				OutputBuffer[i] = (x[i].real * x[i].real) + (x[i].imag * x[i].imag);
  			}
  			
 	 		HPI_Block.ReadyFlag = 1; 	// data ready for host
 	 		
 	 		while(HPI_Block.ReadyFlag)	// wait for host to read it
 	 			;
   			
		
 		}	
  			
   	}   
}

void fft_c(int n, COMPLEX *x, COMPLEX *W)
///////////////////////////////////////////////////////////////////////
// Purpose:   Calculate the radix-2 decimation-in-time FFT.
//
// Input:     n: length of FFT, x: input array of complex numbers,
//            W: array of precomputed twiddle factors
//
// Returns:   values in array x are replaced with result
//
// Calls:     Nothing
//
// Notes:     Bit-reversed address reordering of the sequence 
//            is performed in this function.
///////////////////////////////////////////////////////////////////////
{
    COMPLEX u, temp, tm;
    COMPLEX *Wptr;

    int i, j, k, len, Windex; 
    
    // perform fft butterfly
    Windex = 1;
    for(len = n/2 ; len > 0 ; len /= 2) {
        Wptr = W;
        for (j = 0 ; j < len ; j++) {
            u = *Wptr;
            for (i = j ; i < n ; i = i + 2*len) {
                temp.real = x[i].real + x[i+len].real;
                temp.imag = x[i].imag + x[i+len].imag;
                tm.real = x[i].real - x[i+len].real;
                tm.imag = x[i].imag - x[i+len].imag;             
                x[i+len].real = tm.real*u.real - tm.imag*u.imag;
                x[i+len].imag = tm.real*u.imag + tm.imag*u.real;
                x[i] = temp;
            }
            Wptr = Wptr + Windex;
        }
        Windex = 2*Windex;
    }
    
    // rearrange data by bit reversed addressing 
    // this step must occur after the fft butterfly
    j = 0;
    for (i = 1; i < (n-1); i++) {
        k = n/2;
        while(k <= j) {
            j -= k;
            k /= 2;
        }
        j += k;
        if (i < j) {
            temp = x[j];
            x[j] = x[i];
            x[i] = temp;
        }
    }
    
}  // end of fft_c function

void init_W(int n, COMPLEX *W)
///////////////////////////////////////////////////////////////////////
// Purpose:   Calculate the twiddle factors needed by the FFT.
//
// Input:     n: length of FFT, W: array to store twiddle factors
//
// Returns:   values are stored in array W
//
// Calls:     Nothing
//
// Notes:     Floats used rather than double as this is intended
//            for a DSP CPU target.  Could change to double.
///////////////////////////////////////////////////////////////////////
{
    int i;

    float a = 2.0*PI/n;

    for(i = 0 ; i < n ; i++) {
        W[i].real = (float) cos(-i*a);
        W[i].imag = (float) sin(-i*a);
    }
}
